/*
 * Author     Pascal Lindelauf
 * E-Mail     plindelauf@yellowtail.nl
 * Copyright  2007 Yellowtail
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the GNU Public License v2.0 or any later version
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.txt
 */
/*
 * This class contains operations that simplify access to  
 * UML class elements.
 */
package com.yellowtail.uml14.gen.csharp.services;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;

import fr.obeo.acceleo.ecore.factories.FactoryException;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.uml14.foundation.core.AssociationEnd;
import fr.obeo.acceleo.uml14.foundation.core.Attribute;
import fr.obeo.acceleo.uml14.foundation.core.Class;
import fr.obeo.acceleo.uml14.foundation.core.Classifier;
import fr.obeo.acceleo.uml14.foundation.core.Feature;
import fr.obeo.acceleo.uml14.foundation.core.Generalization;
import fr.obeo.acceleo.uml14.foundation.core.Namespace;
import fr.obeo.acceleo.uml14.foundation.core.Operation;
import fr.obeo.acceleo.uml14.foundation.core.TaggedValue;
import fr.obeo.acceleo.uml14.foundation.data_types.MultiplicityRange;

/**
 * @author pli
 *
 */

public class ClassService {	
	private AssociationService associationService = new AssociationService();
	private AttributeService attributeService = new AttributeService();
	private NamespaceService namespaceService = new NamespaceService();
	private StringService stringService = new StringService();
	private Uml14Service uml14Service = new Uml14Service();
	private Uml14TypeResolver uml14TypeResolver = new Uml14TypeResolver();
	
	public String className(Class cls) throws ENodeCastException{
		return cls.getName();
	}

	public Collection children(Class cls) throws ENodeCastException{
		Collection childs = new ArrayList(); 
		
		for (Iterator i = cls.getSpecialization().iterator(); i.hasNext();){
			childs.add((Class)((Generalization)i.next()).getChild());
		}
		return childs;
	}

	public boolean isChild(Classifier cls) throws ENodeCastException{
		return (cls.getGeneralization().size()>0);
	}

	public String XLevelAndDownClassPath(Class cls, int level, String seperator) throws ENodeCastException{
		return namespaceService.XLevelAndDownNamespacePath(cls.getNamespace(), level, seperator);
	}

	public String pluralName(Class cls) throws ENodeCastException{
		String name = cls.getName();
		
		if (name.substring(name.length()).equalsIgnoreCase("s")){
			name += "es";
		}else{
			name += "s";
		}
		return name;
	}

	public boolean containsACollectionRef(Class cls) throws ENodeCastException {
		Collection attributes = getAttributes(cls);
		Attribute att = null;
		boolean collectionFound = false;
		
		for (Iterator i = attributes.iterator(); i.hasNext() && !collectionFound;){
			att = (Attribute)i.next();
			collectionFound = (((MultiplicityRange)att.getMultiplicity().getRange().get(0)).getUpper() == -1);
		}
		return collectionFound;
	}

	public Collection getAttributes(Class cls) throws ENodeCastException {
		Collection attributes = new ArrayList();
		EList features = cls.getFeature();
		Feature feature = null;
		
		for (Iterator i = features.iterator(); i.hasNext();){
			feature = (Feature)i.next();
			if (feature.getClass().getName().toLowerCase().indexOf("attribute") != -1){
				attributes.add((Attribute)feature);
			}
		}
		return attributes;
	}
	
	public Collection getOperations(Class cls) throws ENodeCastException {
		Collection operations = new ArrayList();
		EList features = cls.getFeature();
		Feature feature = null;
		
		for (Iterator i = features.iterator(); i.hasNext();){
			feature = (Feature)i.next();
			if (feature.getClass().getName().toLowerCase().indexOf("operation") != -1){
				operations.add((Operation)feature);
			}
		}
		return operations;
	}
	
	public Collection attribute(Class cls) throws ENodeCastException {
		return getAttributes(cls);
	}

	public Class parent(Class cls) throws ENodeCastException {
		EList generalizations = cls.getGeneralization();
		
		if (generalizations.size() == 0){
			return null;
		}else{
			return (Class)((Generalization)generalizations.get(0)).getParent();
		}
	}
	
	public Class rootParent(Class cls) throws ENodeCastException {
		EList generalizations = cls.getGeneralization();
		
		if (generalizations.size() == 0){
			return cls;
		}else{
			return rootParent((Class)((Generalization)generalizations.get(0)).getParent());
		}
	}
	
	public Attribute getIdentifier(Class cls) throws ENodeCastException {
		// TBD: Should be solved with a marking or attribute stereotype
		Collection attributes = getAttributes(cls);
		Attribute att = null;
		boolean identifierFound = false;
		
		for (Iterator i = attributes.iterator(); i.hasNext() && !identifierFound;){
			att = (Attribute)i.next();
			identifierFound = attributeService.isIdentifier(att);
		}
	
		return (identifierFound ? att : null);
	}

	public boolean thisContainsAnIdentifier(Class cls) throws ENodeCastException {
		return getIdentifier(cls) != null;
	}

	public boolean thisOrParentContainsAnIdentifier(Class cls) throws ENodeCastException {
		if (thisContainsAnIdentifier(cls)){
			return true;
		}else{
			if (((Class)cls).getGeneralization().size() == 0){
				return false;
			}else{
				return thisOrParentContainsAnIdentifier((Class)((Generalization)cls.getGeneralization().get(0)).getParent());
			}
		}
	}

	public Collection referencedNamespaces(Class cls, Namespace currentNamespace) throws ENodeCastException, FactoryException {
		EList associations = cls.getAssociation();
		Collection attributes = getAttributes(cls);
		Collection referencedNamespaceCollection = new ArrayList();
		
		// First check all associations 
		for (Iterator i = associations.iterator(); i.hasNext(); ){
			AssociationEnd assocEnd = (AssociationEnd)i.next();
			AssociationEnd targetAssocEnd = associationService.targetAssociationEnd(assocEnd);
			if (!currentNamespace.equals((Namespace)targetAssocEnd.getParticipant().getNamespace()) &&
				 targetAssocEnd.isIsNavigable() && 
				 (uml14Service.hasStereotype(targetAssocEnd.getParticipant(), "entityObject") ||
				  uml14Service.hasStereotype(targetAssocEnd.getParticipant(), "valueObject"))){
				if (!referencedNamespaceCollection.contains((Namespace)targetAssocEnd.getParticipant().getNamespace())){
					referencedNamespaceCollection.add((Namespace)targetAssocEnd.getParticipant().getNamespace());
				}
			}
		}

		// Then check all attributes with "foreign" types 
		for (Iterator i = attributes.iterator(); i.hasNext(); ){
			Attribute att = (Attribute)i.next();

			if (!currentNamespace.equals((Namespace)att.getType().getNamespace()) &&
				 (uml14Service.hasStereotype(att.getType(), "entityObject") ||
				  uml14Service.hasStereotype(att.getType(), "valueObject") ||
				  uml14Service.hasStereotype(att.getType(), "enumeration"))){
				if (!referencedNamespaceCollection.contains((Namespace)att.getType().getNamespace())){
					referencedNamespaceCollection.add((Namespace)att.getType().getNamespace());
				}
			}
		}
		
		return referencedNamespaceCollection;
	}
	
	public boolean isAggregateRoot(Class cls) throws ENodeCastException {
		EList taggedValues = cls.getTaggedValue();
		boolean isAggregateRootValue = false;
		
		for (Iterator i = taggedValues.iterator(); i.hasNext() && !isAggregateRootValue;){
			TaggedValue t = (TaggedValue)i.next();
			if (t.getType().getName().equalsIgnoreCase("isAggregateRoot")){
				isAggregateRootValue = t.getDataValue().equalsIgnoreCase("true");
			}
		}
		return isAggregateRootValue;
	}	

	public String tableName(Class cls) throws ENodeCastException {
		return "tbl" + cls.getName();
	}
	
	public String foreignKeyName(Class cls, String roleName) throws ENodeCastException {
		Attribute identifier = this.getIdentifier(cls);
		String idPrefix = "";
		
		if (!roleName.equals("")){
			idPrefix = roleName;
		}else{
			idPrefix = stringService.firstLower(cls.getName());
		}
		
		if (identifier !=null){
			return idPrefix + stringService.firstUpper(identifier.getName());
		}else{
			return idPrefix + "Id";
		}
	}

	public String foreignKeyTypeName(Class cls) throws ENodeCastException {
		Attribute identifier = this.getIdentifier(cls);
		
		if (identifier !=null){
			return uml14TypeResolver.resolveToSQLServer(identifier.getType());
		}else{
			return "int";
		}
	}
}