/*
 * Author     Pascal Lindelauf
 * E-Mail     plindelauf@yellowtail.nl
 * Copyright  2007 Yellowtail
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the GNU Public License v2.0 or any later version
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.txt
 */
/*
 * This class contains operations that simplify access to  
 * UML operation elements.
 */
package com.yellowtail.uml14.gen.csharp.services;

import java.util.Collection;
import java.util.Iterator;

import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.uml14.foundation.core.Class;
import fr.obeo.acceleo.uml14.foundation.core.Operation;
import fr.obeo.acceleo.uml14.foundation.core.Parameter;
import fr.obeo.acceleo.uml14.foundation.data_types.VisibilityKind;

/**
 * @author pli
 *
 */

public class OperationService {
	
	private ClassService classService = new ClassService();
	
	public String visibilityName(Operation op) throws ENodeCastException {
		switch (op.getVisibility().getValue()){
			case VisibilityKind.VK_PRIVATE: 
				return "private";
			case VisibilityKind.VK_PROTECTED:
				return "protected";
			case VisibilityKind.VK_PUBLIC:
				return "public";
			default:
				return "public";
		}
	}

	/*&
	 * @param op
	 * @return true if the current operation is overwriting an operation on one of its base classes.
	 *         Note: we assume that we deal with single inheritance only.
	 * @throws ENodeCastException
	 */
	public boolean isOverwriting(Operation op) throws ENodeCastException {
		boolean similarOperationFound = false;
		Collection operations;
		Class parentClass;

		if (op.getOwner().getGeneralization().size() == 0){
			parentClass = null;
		}else{
			parentClass = classService.parent((Class)op.getOwner());
		}
		
		while (parentClass != null && !similarOperationFound){
			operations = classService.getOperations(parentClass);
			
			for (Iterator i = operations.iterator(); i.hasNext() && !similarOperationFound;){
				similarOperationFound = OneOverridesTheOther(op, (Operation)i.next());
			}
		}
		
		return (similarOperationFound);
	}

	/*&
	 * @param op
	 * @return true if the current operation is overloading an operation on one of its base classes.
	 *         Note: we assume that we deal with single inheritance only.
	 * @throws ENodeCastException
	 */
	public boolean isOverloading(Operation op) throws ENodeCastException {
		boolean similarOperationFound = false;
		Collection operations;
		Class parentClass;

		if (op.getOwner().getGeneralization().size() == 0){
			parentClass = null;
		}else{
			parentClass = classService.parent((Class)op.getOwner());
		}
		
		while (parentClass != null && !similarOperationFound){
			operations = classService.getOperations(parentClass);
			
			for (Iterator i = operations.iterator(); i.hasNext() && !similarOperationFound;){
				similarOperationFound = OneOverloadsTheOther(op, (Operation)i.next());
			}
		}
		
		return (similarOperationFound);
	}

	private boolean OneOverridesTheOther(Operation op1, Operation op2) {
		boolean areStillSimilar = true;
		Collection parameters1;
		Collection parameters2;
		Parameter parameter1;
		Parameter parameter2;

		areStillSimilar = op1.getName().equals(op2.getName());
		
		if (areStillSimilar){
			areStillSimilar = (op1.getParameter().size() == op2.getParameter().size());
		}
		
		if (areStillSimilar){
			parameters1 = op1.getParameter();
			parameters2 = op2.getParameter();
			
			Iterator i2 = parameters2.iterator();
			for (Iterator i1 = parameters1.iterator(); i1.hasNext() && areStillSimilar;){
				parameter1 = (Parameter)i1.next();
				parameter2 = (Parameter)i2.next();
				
				areStillSimilar = areStillSimilar && (parameter1.getName().equals(parameter2.getName()) && parameter1.getType() == parameter2.getType()); 
			}
		}

		return areStillSimilar;
	}

	private boolean OneOverloadsTheOther(Operation op1, Operation op2) {
		boolean areStillSimilar = true;
		boolean haveDifferentParameters = false;
		Collection parameters1;
		Collection parameters2;
		Parameter parameter1;
		Parameter parameter2;

		areStillSimilar = op1.getName().equals(op2.getName());
		
		if (areStillSimilar){
			areStillSimilar = (op1.getParameter().size() == op2.getParameter().size());
		}
		
		if (areStillSimilar){
			parameters1 = op1.getParameter();
			parameters2 = op2.getParameter();
			
			Iterator i2 = parameters2.iterator();
			for (Iterator i1 = parameters1.iterator(); i1.hasNext() && !haveDifferentParameters;){
				parameter1 = (Parameter)i1.next();
				parameter2 = (Parameter)i2.next();
				
				haveDifferentParameters = haveDifferentParameters || (!parameter1.getName().equals(parameter2.getName()) || parameter1.getType() != parameter2.getType()); 
			}
		}

		return areStillSimilar && haveDifferentParameters;
	}
}