package fr.obeo.weblog.dao.blog.test;

// Start of user code for import 
import org.hibernate.Transaction;
import junit.framework.Assert;
import junit.framework.TestCase;

import fr.obeo.fwk.dao.exception.DaoException;
import fr.obeo.fwk.dao.hibernate.HibernateUtil;
import fr.obeo.weblog.dao.WeblogDaoFactory;
import fr.obeo.weblog.dao.blog.api.IUserDao;
import fr.obeo.weblog.business.blog.User;

// End of user code for import

public class IUserDaoTest extends TestCase {

	private IUserDao dao;
	private Transaction tx;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		dao = WeblogDaoFactory.getUserDao();
		tx = HibernateUtil.currentSession().beginTransaction();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
		tx.commit();
		HibernateUtil.closeSession();
	}

	/**
	 * Test the creation of the entity User.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Search the entity and verify it exist</li></ul>
	 */
	public final void testCreateUser() throws DaoException {
        // fill attributes with example values
		User user = getUserExample();
		
        // Execute the tested code
		dao.createUser(user);

		// Verify result
		boolean found = false;
		for(User currentUser : dao.findAllUsers()) {
			if (currentUser.equals(user)) {
				found = true;
			}
		}
		Assert.assertTrue("User not created", found);
	}

	/**
	 * Test the modification of an entity User.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Modify the entity</li>
	 * <li>Step 3 : Search the entity and verify the modified values</li></ul>
	 */
	public final void testUpdateUser() throws DaoException {
        // Initialized the test
		User user = getUserExample();
		dao.createUser(user);

        // Execute the tested code
		user.setEmail("secondEmail");
		user.setFirstName("secondFirstName");
		user.setLastName("secondLastName");
		user.setLogin("secondLogin");
		user.setPassword("secondPassword");
		dao.updateUser(user);

		// Verify result
		boolean found = false;
		for(User currentUser : dao.findAllUsers()) {
			if (currentUser.equals(user)) {
				found = true;
				Assert.assertEquals("Value email not modified", "secondEmail", user.getEmail());
				Assert.assertEquals("Value firstName not modified", "secondFirstName", user.getFirstName());
				Assert.assertEquals("Value lastName not modified", "secondLastName", user.getLastName());
				Assert.assertEquals("Value login not modified", "secondLogin", user.getLogin());
				Assert.assertEquals("Value password not modified", "secondPassword", user.getPassword());
			}
		}
		Assert.assertTrue("User not found", found);
	}

	/**
	 * Test the suppression of an entity User.<br/>
	 * <ul><li>Step 1 : Create an entity</li>
	 * <li>Step 2 : Delete the entity</li>
	 * <li>Step 3 : Search the entity and verify it doesn't exist anymore</li></ul>
	 */
	public final void testDeleteUser() throws DaoException {
        // Initialized the test
		User user = getUserExample();
		dao.createUser(user);
		
        // Execute the tested code
		dao.deleteUser(user);
		
		// Verify result
		boolean found = false;
		for(User currentUser : dao.findAllUsers()) {
			if (currentUser.equals(user)) {
				found = true;				
			}
		}
		Assert.assertFalse("User not deleted", found);
	}

	/**
	 * Test the search of all entities User.<br/>
	 * <ul><li>Step 1 : Create many entities</li>
	 * <li>Step 2 : Search all entities and verify the entities created are found</li></ul>
	 */
	public final void testFindAllUsers() throws DaoException {
		// Verify number of element before testing
		int before = dao.findAllUsers().size();
		Assert.assertTrue("FindAll must return at least 0", before >= 0);
		
        // Add two elements
		User user1 = getUserExample();
		dao.createUser(user1);
				
		User user2 = getUserExample();
		dao.createUser(user2);

		// Verify result
		int after = dao.findAllUsers().size();
		Assert.assertEquals(
				"FindAll don't find the good number of elements", 
				2,
				after - before);
	}



	/**
	 * Create an example of User
	 */
	protected User getUserExample() throws DaoException {
		// fill attributes with example values
		User user = new User();
		user.setEmail("myEmail");
		user.setFirstName("myFirstName");
		user.setLastName("myLastName");
		user.setLogin("myLogin");
		user.setPassword("myPassword");
		
		return user;
	}
	
	/**
	 * Create a second example of User with other values
	 */
	protected User getUserExample2() throws DaoException {
		// fill attributes with example values
		User user = new User();
		user.setEmail("secondEmail");
		user.setFirstName("secondFirstName");
		user.setLastName("secondLastName");
		user.setLogin("secondLogin");
		user.setPassword("secondPassword");
		
		return user;
	}
	
	// Start of user code of specific tests
	// End of user code of specific tests

}
